@@ -24,14 +24,14 @@ from gi.repository import GLib
 import cairo
 import copy
 import hashlib
-import mlt
+import json
+try:
+    import mlt7 as mlt
+except:
+    import mlt
 import os
-from os import listdir
-from os.path import isfile, join
-import pickle
 import subprocess
 import shutil
-import re
 import sys
 import threading
 import time
@@ -39,12 +39,16 @@ import time
 import appconsts
 import atomicfile
 import blenderheadless
+import ccrutils
 import dialogutils
 import edit
 import editorstate
 from editorstate import current_sequence
 from editorstate import PROJECT
+import fluxity
+import fluxityheadless
 import gui
+import guiutils
 import gmicheadless
 import gmicplayer
 import jobs
@@ -55,9 +59,18 @@ import rendergui
 import respaths
 import simpleeditors
 import toolsencoding
+import updater
 import userfolders
 import utils
 
+"""
+This module creates <ConatainerClipType>Actions wrapper objects for container clips data that are used to execute
+all actions on container clips data.
+
+Objects of class containerclips.ContainerData are persistent data for container clips, 
+objects in this module created and discarded as needed.
+"""
+
 FULL_RENDER = 0
 CLIP_LENGTH_RENDER = 1
 PREVIEW_RENDER = 2
@@ -67,9 +80,13 @@ OVERLAY_COLOR = (0.17, 0.23, 0.63, 0.5)
 GMIC_TYPE_ICON = None
 MLT_XML_TYPE_ICON = None
 BLENDER_TYPE_ICON = None
+FLUXITY_TYPE_ICON = None
 
 NEWLINE = '\n'
 
+set_plugin_to_be_edited_func = None
+get_edited_plugin_clip = None
+
 # ----------------------------------------------------- interface
 def get_action_object(container_data):
     if container_data.container_type == appconsts.CONTAINER_CLIP_GMIC:
@@ -78,11 +95,13 @@ def get_action_object(container_data):
          return MLTXMLContainerActions(container_data)
     elif container_data.container_type == appconsts.CONTAINER_CLIP_BLENDER:
          return BlenderContainerActions(container_data)
-
+    elif container_data.container_type == appconsts.CONTAINER_CLIP_FLUXITY:
+         return FluxityContainerActions(container_data)
+         
 # ------------------------------------------------------------ thumbnail creation helpers
 def _get_type_icon(container_type):
     # TODO: When we get third move this into action objects.
-    global GMIC_TYPE_ICON, MLT_XML_TYPE_ICON, BLENDER_TYPE_ICON
+    global GMIC_TYPE_ICON, MLT_XML_TYPE_ICON, BLENDER_TYPE_ICON, FLUXITY_TYPE_ICON
     
     if GMIC_TYPE_ICON == None:
         GMIC_TYPE_ICON = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + "container_clip_gmic.png")
@@ -90,6 +109,8 @@ def _get_type_icon(container_type):
         MLT_XML_TYPE_ICON = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + "container_clip_mlt_xml.png")
     if BLENDER_TYPE_ICON == None:
         BLENDER_TYPE_ICON = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + "container_clip_blender.png")
+    if FLUXITY_TYPE_ICON == None:
+        FLUXITY_TYPE_ICON = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + "container_clip_fluxity.png")
         
     if container_type == appconsts.CONTAINER_CLIP_GMIC:
         return GMIC_TYPE_ICON
@@ -97,7 +118,9 @@ def _get_type_icon(container_type):
         return MLT_XML_TYPE_ICON
     elif container_type == appconsts.CONTAINER_CLIP_BLENDER:  
         return BLENDER_TYPE_ICON
-
+    elif container_type == appconsts.CONTAINER_CLIP_FLUXITY:  
+        return FLUXITY_TYPE_ICON
+        
 def _write_thumbnail_image(profile, file_path, action_object):
     """
     Writes thumbnail image from file producer
@@ -113,8 +136,6 @@ def _write_thumbnail_image(profile, file_path, action_object):
 
     # Create one frame producer
     producer = mlt.Producer(profile, str(file_path))
-    if producer.is_valid() == False:
-        raise ProducerNotValidError(file_path)
 
     info = utils.get_file_producer_info(producer)
 
@@ -167,7 +188,6 @@ class AbstractContainerActionObject:
         self.render_type = FULL_RENDER
         self.clip = clip
         self.launch_render_data = (clip, 0, self.container_data.unrendered_length, 0)
-
         job_proxy = self.get_launch_job_proxy()
         jobs.add_job(job_proxy)
         
@@ -226,7 +246,7 @@ class AbstractContainerActionObject:
             return self.container_data.render_data.render_dir + appconsts.CC_PREVIEW_RENDER_DIR
 
     def get_container_program_id(self):
-        id_md_str = str(self.container_data.container_clip_uid) + str(self.container_data.container_type) + self.container_data.program + self.container_data.unrendered_media #
+        id_md_str = str(self.container_data.container_clip_uid) + str(self.container_data.container_type) + self.container_data.program + self.container_data.unrendered_media
         return hashlib.md5(id_md_str.encode('utf-8')).hexdigest() 
 
     def get_container_thumbnail_path(self):
@@ -276,9 +296,17 @@ class AbstractContainerActionObject:
             print("No frame file found for container clip at:", self.get_rendered_media_dir())
             return None
 
-        resource_name_str = utils.get_img_seq_resource_name(frame_file, True)
+        resource_name_str = utils.get_img_seq_resource_name(frame_file)
         return self.get_rendered_media_dir() + "/" + resource_name_str
 
+    def get_rendered_video_clip_path(self):
+        if self.container_data.render_data.save_internally == True:
+            resource_path = self.get_session_dir() +  "/" + appconsts.CONTAINER_CLIP_VIDEO_CLIP_NAME + self.container_data.render_data.file_extension
+        else:
+            resource_path = self.container_data.render_data.render_dir +  "/" + self.container_data.render_data.file_name + self.container_data.render_data.file_extension
+    
+        return resource_path
+    
     def get_rendered_thumbnail(self):
         thumbnail_path = self.get_container_thumbnail_path()
         if os.path.isfile(thumbnail_path) == True:
@@ -300,17 +328,13 @@ class AbstractContainerActionObject:
             resource_path = self.get_rendered_frame_sequence_resource_path()
             if resource_path == None:
                 return # TODO: User info?
+                
+            rendered_clip = current_sequence().create_file_producer_clip(resource_path, new_clip_name=self.clip.name, novalidate=False, ttl=1)
 
-            rendered_clip = current_sequence().create_file_producer_clip(resource_path, new_clip_name=None, novalidate=False, ttl=1)
-            
         # Using video clip as clip
         else:
-            if self.container_data.render_data.save_internally == True:
-                resource_path = self.get_session_dir() +  "/" + appconsts.CONTAINER_CLIP_VIDEO_CLIP_NAME + self.container_data.render_data.file_extension
-            else:
-                resource_path = self.container_data.render_data.render_dir +  "/" + self.container_data.render_data.file_name + self.container_data.render_data.file_extension
-
-            rendered_clip = current_sequence().create_file_producer_clip(resource_path, new_clip_name=None, novalidate=True, ttl=1)
+            resource_path = self.get_rendered_video_clip_path()
+            rendered_clip = current_sequence().create_file_producer_clip(resource_path, new_clip_name=self.clip.name, novalidate=True, ttl=1)
         
         track, clip_index = current_sequence().get_track_and_index_for_id(self.clip.id)
         
@@ -336,8 +360,11 @@ class AbstractContainerActionObject:
             self.clip.container_data.last_render_type = CLIP_LENGTH_RENDER
             action = edit.container_clip_clip_render_replace(data)
             action.do_edit()
-                
-    def set_video_endoding(self, clip):
+
+        return rendered_clip
+
+    def set_video_endoding(self, clip, callback=None):
+        self.external_encoding_callback = callback
         current_profile_index = mltprofiles.get_profile_index_for_profile(current_sequence().profile)
         # These need to re-initialized always when using this module.
         toolsencoding.create_widgets(current_profile_index, True, True)
@@ -366,11 +393,12 @@ class AbstractContainerActionObject:
     def encode_settings_callback(self, dialog, response_id):
         if response_id == Gtk.ResponseType.ACCEPT:
             self.container_data.render_data = toolsencoding.get_render_data_for_current_selections()
+            if self.external_encoding_callback != None:
+                self.external_encoding_callback(self.container_data.render_data)
 
         dialog.destroy()
         
     def clone_clip(self, old_clip):
-
         new_container_data = copy.deepcopy(old_clip.container_data)
         new_container_data.generate_clip_id()
         
@@ -396,6 +424,7 @@ class AbstractContainerActionObject:
             # we have rendered a frame sequence clip for media last.
             old_frames_dir = container_clip_action_object.get_session_dir() + appconsts.CC_RENDERED_FRAMES_DIR
             new_frames_dir = self.get_session_dir() + appconsts.CC_RENDERED_FRAMES_DIR
+            os.rmdir(new_frames_dir)
             shutil.copytree(old_frames_dir, new_frames_dir)
         
             resource_path = self.get_rendered_frame_sequence_resource_path()
@@ -410,13 +439,6 @@ class AbstractContainerActionObject:
         
     def _build_icon(self, icon_path):
         cr, surface = _create_image_surface(icon_path)
-        cr.rectangle(0, 0, appconsts.THUMB_WIDTH, appconsts.THUMB_HEIGHT)
-        cr.set_source_rgba(*OVERLAY_COLOR)
-        cr.fill()
-        type_icon = _get_type_icon(self.container_data.container_type)
-        cr.set_source_surface(type_icon, 1, 30)
-        cr.set_operator (cairo.OPERATOR_OVERLAY)
-        cr.paint_with_alpha(0.5)
         return surface
         
     def load_icon(self):
@@ -425,6 +447,8 @@ class AbstractContainerActionObject:
     def edit_program(sef, clip):
         print("AbstractContainerActionObject.edit_program not impl")
 
+    def create_image_surface(self, icon_path):
+        return _create_image_surface(icon_path)
 
 class GMicContainerActions(AbstractContainerActionObject):
 
@@ -446,7 +470,6 @@ class GMicContainerActions(AbstractContainerActionObject):
             command_list.append("-output")
             command_list.append(test_out_file)
 
-
             # Render preview and write log
             FLOG = open(userfolders.get_cache_dir() + "gmic_container_validating_log", 'w')
             p = subprocess.Popen(command_list, stdin=FLOG, stdout=FLOG, stderr=FLOG)
@@ -510,8 +533,9 @@ class GMicContainerActions(AbstractContainerActionObject):
         subprocess.Popen(command_list)
         
     def update_render_status(self):
-        
-        Gdk.threads_enter()
+        GLib.idle_add(self._do_update_render_status)
+            
+    def _do_update_render_status(self):
                     
         if gmicheadless.session_render_complete(self.get_container_program_id()) == True:
             
@@ -555,7 +579,7 @@ class GMicContainerActions(AbstractContainerActionObject):
                         job_msg.progress = 1.0
 
                     if job_msg.progress > 1.0:
-                        # Ffix how progress is calculated in gmicheadless because producers can render a bit longer then required.
+                        # Fix how progress is calculated in gmicheadless because producers can render a bit longer then required.
                         job_msg.progress = 1.0
 
                 job_msg.elapsed = float(elapsed)
@@ -565,10 +589,7 @@ class GMicContainerActions(AbstractContainerActionObject):
             else:
                 pass # This can happen sometimes before gmicheadless.py has written a status message, we just do nothing here.
 
-        Gdk.threads_leave()
-
     def abort_render(self):
-        #self.remove_as_status_polling_object()
         gmicheadless.abort_render(self.get_container_program_id())
 
     def create_icon(self):
@@ -581,9 +602,247 @@ class GMicContainerActions(AbstractContainerActionObject):
         cr.set_source_surface(type_icon, 1, 30)
         cr.set_operator (cairo.OPERATOR_OVERLAY)
         cr.paint_with_alpha(0.5)
- 
         return (surface, length, icon_path)
+
+
+class FluxityContainerActions(AbstractContainerActionObject):
+
+    def __init__(self, container_data):
+        AbstractContainerActionObject.__init__(self, container_data)
+        self.do_filters_clone = True
+        self.plugin_create_render_complete_callback = None # set at object creation site if needed
+
+    def validate_program(self):
+        try:
+            script_file = open(self.container_data.program)
+            user_script = script_file.read()
+            profile_file_path = mltprofiles.get_profile_file_path(current_sequence().profile.description())
+            fctx = fluxity.render_preview_frame(user_script, script_file, 0, None, profile_file_path)
+         
+            if fctx.error == None:
+                data_json = fctx.get_script_data()
+                self.container_data.data_slots["fluxity_plugin_edit_data"] = json.loads(data_json) # script data saved as Python object, not json str.
+                return (True, None) # no errors
+            else:
+                return (False,  fctx.error)
+    
+        except Exception as e:
+            return (False, str(e))
+    
+    def re_render_screenshot(self):
+        script_file = open(self.container_data.program)
+        user_script = script_file.read()
+        profile_file_path = mltprofiles.get_profile_file_path(current_sequence().profile.description())
+        frame = self.container_data.unrendered_length // 2
+        screenshot_file = self.get_container_thumbnail_path()
         
+        fctx = fluxity.render_preview_frame(user_script, script_file, frame, None, profile_file_path)
+        fctx.priv_context.frame_surface.write_to_png(screenshot_file)
+        cr, surface = _create_image_surface(screenshot_file)
+        return (screenshot_file, surface)
+
+    def get_job_proxy(self):
+        job_proxy = jobs.JobProxy(self.get_container_program_id(), self)
+        job_proxy.type = jobs.CONTAINER_CLIP_RENDER_FLUXITY
+        return job_proxy
+
+    def get_job_name(self):
+        data_object = self.container_data.data_slots["fluxity_plugin_edit_data"] 
+        return data_object["name"]
+
+    def _launch_render(self, clip, range_in, range_out, unused_frame_ofset):
+        self.create_data_dirs_if_needed()
+        
+        range_out = range_out + 1 # MLT handles out frames inclusive but fluxity exlusive.
+                                  # We just need to manually make sure we get always correct lengths.
+        
+        self.render_range_in = range_in
+        self.render_range_out = range_out
+ 
+        fluxityheadless.clear_flag_files(self.get_container_program_id())
+    
+        # We need data to be available for render process, 
+        # create video_render_data object with default values if not available.
+        if self.container_data.render_data == None:
+            self.container_data.render_data = toolsencoding.create_container_clip_default_render_data_object(current_sequence().profile)
+            self.container_data.render_data.do_video_render = False 
+
+        fluxityheadless.set_render_data(self.get_container_program_id(), self.container_data.render_data)
+        
+        job_msg = self.get_job_queue_message()
+        job_msg.text = _("Render Starting...")
+        job_msg.status = jobs.RENDERING
+        jobs.update_job_queue(job_msg)
+
+        # we could drop sending args if we wanted and just use this. 
+        ccrutils.write_misc_session_data(self.get_container_program_id(), "fluxity_plugin_edit_data", self.container_data.data_slots["fluxity_plugin_edit_data"])
+        
+        args = ("session_id:" + self.get_container_program_id(), 
+                "script:" + str(self.container_data.program),
+                "range_in:" + str(range_in),
+                "range_out:"+ str(range_out),
+                "profile_desc:" + PROJECT().profile.description().replace(" ", "_"))  # Here we have our own string space handling, maybe change later..
+
+        # Create command list and launch process.
+        command_list = [sys.executable]
+        command_list.append(respaths.LAUNCH_DIR + "flowbladefluxityheadless")
+        for arg in args:
+            command_list.append(arg)
+
+        subprocess.Popen(command_list)
+        
+    def update_render_status(self):
+        GLib.idle_add(self._do_update_render_status)
+            
+    def _do_update_render_status(self):
+        
+        # We need to set these None so that when render is complete tlinewidgets.py drawing code no longer thinks
+        # that render is in progress
+        self.container_data.progress = None
+        self.clip.container_data.progress = None
+        
+        if fluxityheadless.session_render_complete(self.get_container_program_id()) == True:
+            job_msg = self.get_completed_job_message()
+            jobs.update_job_queue(job_msg)
+            
+            if self.plugin_create_render_complete_callback == None:
+                # Completed render for timeline container clip update is handeled here.
+                GLib.idle_add(self.plugin_tline_render_comlete)
+                GLib.idle_add(self.create_producer_and_do_update_edit, None)
+            else:
+                # Completed render for adding Generator plugin as rendered video clip is handeled here. 
+                if self.container_data.render_data.do_video_render == False:
+                    resource_path = self.get_rendered_frame_sequence_resource_path()
+                else:
+                    resource_path = self.get_rendered_video_clip_path()
+
+                GLib.idle_add(self.plugin_create_render_complete_callback, resource_path, self.container_data)
+
+        else:
+            status = fluxityheadless.get_session_status(self.get_container_program_id())
+            if status != None:
+                step, frame, length, elapsed = status
+
+                steps_count = 2
+                if self.container_data.render_data.do_video_render == False:
+                    steps_count = 1
+                msg = _("Step ") + str(step) + " / " + str(steps_count) + " - "
+                if step == "1":
+                    msg += _("Writing Clip Frames")
+                else:
+                     msg += _("Encoding Video")
+                
+                msg += " - " + self.get_job_name()
+                
+                job_msg = self.get_job_queue_message()
+                if self.render_type == FULL_RENDER:
+                    job_msg.progress = float(frame)/float(length)
+                else:
+                    if step == "1":
+                        render_length = self.render_range_out - self.render_range_in
+                    job_msg.progress = float(frame)/float(render_length)
+                    
+                    if job_msg.progress < 0.0:
+                        # hack to fix how gmiplayer.FramesRangeWriter works.
+                        # We would need to patch to G'mic Tool to not need this but this is easier.
+                        job_msg.progress = 1.0
+
+                    if job_msg.progress > 1.0:
+                        # Fix how progress is calculated in rendering process because producers can render a bit longer then required.
+                        job_msg.progress = 1.0
+
+                job_msg.elapsed = float(elapsed)
+                job_msg.text = msg
+                
+                jobs.update_job_queue(job_msg)
+
+                # Update tline render % display, we need to set different attrs for 
+                # first and succeeding renders.
+                self.container_data.progress = job_msg.progress
+                self.clip.container_data.progress = job_msg.progress
+                updater.repaint_tline()
+                
+            else:
+                pass # This can happen sometimes before gmicheadless.py has written a status message, we just do nothing here.
+
+
+    def plugin_tline_render_comlete(self):
+        clip = self.create_producer_and_do_update_edit(None)
+        # Reopen in edit panel, doing callback to avoid circular imports
+        set_plugin_to_be_edited_func(clip, self)
+
+    def abort_render(self):
+        fluxityheadless.abort_render(self.get_container_program_id())
+
+    def create_icon(self):
+        if self.container_data.data_slots["icon_file"] == None:
+            icon_path, not_used_length, info = _write_thumbnail_image(PROJECT().profile, self.container_data.unrendered_media, self)
+        else:
+            icon_path, not_used_length, info = _write_thumbnail_image(PROJECT().profile, self.container_data.data_slots["icon_file"], self)
+
+        cr, surface = _create_image_surface(icon_path)
+ 
+        data_object = self.container_data.data_slots["fluxity_plugin_edit_data"]
+        length = data_object["length"]
+
+        return (surface, length, icon_path)
+
+    def edit_program(self, clip):
+        set_plugin_to_be_edited_func(clip, self)
+        gui.editor_window.edit_multi.set_visible_child_name(appconsts.EDIT_MULTI_PLUGINS)
+
+    def apply_editors(self, editors):
+        new_editors_list = self.get_editors_data_as_editors_list(editors)
+        self.container_data.data_slots["fluxity_plugin_edit_data"]["editors_list"] = new_editors_list
+        get_edited_plugin_clip().container_data.data_slots["fluxity_plugin_edit_data"]["editors_list"] = new_editors_list
+
+    def render_fluxity_preview(self, callbacks, editors, preview_frame):
+        self.create_data_dirs_if_needed() # This could be first time we are writing
+                                          # data related to this container clip to disk.
+        
+        completed_callback, error_callback = callbacks
+        new_editors_list = self.get_editors_data_as_editors_list(editors)
+        editors_data_json = json.dumps(new_editors_list)
+        script_file = open(self.container_data.program)
+        user_script = script_file.read()
+        profile_file_path = mltprofiles.get_profile_file_path(current_sequence().profile.description())
+        
+        self.container_data.render_data = toolsencoding.create_container_clip_default_render_data_object(current_sequence().profile)
+        self.container_data.render_data.do_video_render = False 
+        
+        out_folder = self.get_preview_media_dir()
+        if not os.path.exists(out_folder):
+            os.mkdir(out_folder)
+
+        fctx = fluxity.render_preview_frame(user_script, script_file, preview_frame, out_folder, profile_file_path, editors_data_json)
+        if fctx.error != None:
+            error_callback(fctx.error)
+            return
+                    
+        fctx.priv_context.write_out_frame(True)
+        
+        completed_callback()
+
+    def get_editors_data_as_editors_list(self, editor_widgets):
+        new_editors_list = [] # This is the editors list in format created in
+                              # fluxity.FluxityContext.get_script_data()
+        for editor in editor_widgets:
+            value = editor.get_value()
+            # Special casing required for editors that have different internal representation of data
+            # compared to what they give to out scripts.
+            # BE VERY CAREFUL IF ADDING ANY NEW SUCH EDITORS.
+            if editor.editor_type == simpleeditors.SIMPLE_EDITOR_COLOR:
+                value = editor.get_value_as_color_tuple()
+            elif editor.editor_type == simpleeditors.SIMPLE_EDITOR_FLOAT_RANGE or editor.editor_type == simpleeditors.SIMPLE_EDITOR_INT_RANGE:
+                value = editor.get_value_as_range_tuple()
+            elif editor.editor_type == simpleeditors.SIMPLE_EDITOR_OPTIONS:
+                value = editor.get_value_as_tuple()
+
+            new_editor = [editor.id_data, editor.editor_type, value]
+            new_editors_list.append(new_editor)
+        
+        return new_editors_list
+
 
 class MLTXMLContainerActions(AbstractContainerActionObject):
 
@@ -640,8 +899,9 @@ class MLTXMLContainerActions(AbstractContainerActionObject):
         subprocess.Popen(command_list)
 
     def update_render_status(self):
-
-        Gdk.threads_enter()
+        GLib.idle_add(self._do_update_render_status)
+            
+    def _do_update_render_status(self):
                     
         if mltxmlheadless.session_render_complete(self.get_container_program_id()) == True:
             #self.remove_as_status_polling_object()
@@ -658,9 +918,9 @@ class MLTXMLContainerActions(AbstractContainerActionObject):
                 fraction, elapsed = status
 
                 if self.container_data.render_data.do_video_render == True:
-                    msg = _("Rendering Video")
-                elif step == "2":
-                    msg = _("Rendering Image Sequence")
+                    msg = _("Video for: ") + self.clip.name 
+                #elif step == "2":
+                #    msg = _("Image Sequence for: ") + self.clip.name 
 
                 job_msg = self.get_job_queue_message()
                 job_msg.progress = float(fraction)
@@ -670,9 +930,7 @@ class MLTXMLContainerActions(AbstractContainerActionObject):
                 jobs.update_job_queue(job_msg)
             else:
                 pass # This can happen sometimes before gmicheadless.py has written a status message, we just do nothing here.
-                
-        Gdk.threads_leave()
-                
+
     def create_icon(self):
         return self._create_icon_default_action()
 
@@ -745,7 +1003,6 @@ class BlenderContainerActions(AbstractContainerActionObject):
             render_data.do_video_render = False
             render_data.is_preview_render = True
 
-
         if editorstate.app_running_from == editorstate.RUNNING_FROM_FLATPAK:
             render_data.is_flatpak_render = True
         
@@ -816,8 +1073,9 @@ class BlenderContainerActions(AbstractContainerActionObject):
         return render_exec_lines
 
     def update_render_status(self):
-
-        Gdk.threads_enter()
+        GLib.idle_add(self._do_update_render_status)
+            
+    def _do_update_render_status(self):
                     
         if blenderheadless.session_render_complete(self.get_container_program_id()) == True:
 
@@ -848,8 +1106,6 @@ class BlenderContainerActions(AbstractContainerActionObject):
                 jobs.update_job_queue(job_msg)
             else:
                 pass # This can happen sometimes before gmicheadless.py has written a status message, we just do nothing here.
-                
-        Gdk.threads_leave()
 
     def abort_render(self):
         #self.remove_as_status_polling_object()
@@ -907,7 +1163,6 @@ def create_unrendered_clip(length, image_file, data, callback, window_text):
 
 
 # Creates a set length video clip from image to act as container clip unrendered media.
-# NOTE: Currently harcoded to to work on just Blender container clips, expand usage with parameter as needed.
 class UnrenderedCreationThread(threading.Thread):
     
     def __init__(self, length, image_file, data, callback, window_text):
@@ -923,12 +1178,8 @@ class UnrenderedCreationThread(threading.Thread):
         # Image produceer
         img_producer = current_sequence().create_file_producer_clip(str(self.image_file)) # , new_clip_name=None, novalidate=False, ttl=None):
 
-        # Create tractor and track to get right length
-        tractor = mlt.Tractor()
-        multitrack = tractor.multitrack()
-        track0 = mlt.Playlist()
-        multitrack.connect(track0, 0)
-        track0.insert(img_producer, 0, 0, self.length)
+        # Create tractor to get right length.
+        tractor = renderconsumer.get_producer_as_tractor(img_producer, self.length)
     
         # Consumer
         write_file = userfolders.get_cache_dir() + "/unrendered_clip.mp4"
@@ -941,28 +1192,25 @@ class UnrenderedCreationThread(threading.Thread):
         clip_renderer.wait_for_producer_end_stop = True
         clip_renderer.start()
 
-        Gdk.threads_enter()
-        
-        info_text = _("<b>Rendering Placeholder Media For:</b> ")  + self.data.get_program_name() + ".blend"
-
-        progress_bar = Gtk.ProgressBar()
-        dialog = rendergui.clip_render_progress_dialog(None, self.window_text, info_text, progress_bar, gui.editor_window.window, True)
-
-        motion_progress_update = renderconsumer.ProgressWindowThread(dialog, progress_bar, clip_renderer, self.progress_thread_complete)
-        motion_progress_update.start()
-
-        Gdk.threads_leave()
-        
+        GLib.idle_add(self._show_progress_window,  clip_renderer)
+    
         while clip_renderer.stopped == False:       
             time.sleep(0.5)
 
-        Gdk.threads_enter()
+        GLib.idle_add(self._do_write_callback, write_file)
+
+    def _show_progress_window(self, clip_renderer):
         
+        info_text = _("<b>Rendering Placeholder Media For:</b> ")  + self.data.get_program_name()
+
+        progress_bar = Gtk.ProgressBar()
+        self.dialog = rendergui.clip_render_progress_dialog(None, self.window_text, info_text, progress_bar, gui.editor_window.window, True)
+
+        motion_progress_update = guiutils.ProgressWindowThread(self.dialog, progress_bar, clip_renderer, self.progress_thread_complete)
+        motion_progress_update.start()
+        
+    def _do_write_callback(self, write_file):
         self.callback(write_file, self.data)
 
-        Gdk.threads_leave()
-
     def progress_thread_complete(self, dialog, some_number):
-        #  Gdk.threads_enter() is done before this called from "motion_progress_update" thread.
-        dialog.destroy()
-
+        GLib.idle_add(dialogutils.dialog_destroy, self.dialog, None)
