@@ -23,7 +23,7 @@ Module holds current global editor state.
 
 Accessor methods are there mainly to improve code readability elsewhere.
 
-We're using BIG_METHOD_NAMES() for state objects. This is a bit unusual
+We're using BIG_METHOD_NAMES() for some state objects. This is a bit unusual,
 but looks good when reading code.
 """
 
@@ -63,7 +63,7 @@ edit_mode = INSERT_MOVE
 # Ripple Trim tool is ONE_ROLL_TRIM mode + True on this flag
 trim_mode_ripple = False
 
-# Box tool OVERWRITE_MOVE tool mode + True on this flag
+# Box tool is OVERWRITE_MOVE tool mode + True on this flag
 overwrite_mode_box = False
 
 # Media files view filter for selecting displayed media objects in bin
@@ -96,7 +96,7 @@ SCREEN_WIDTH = -1
 # Runtime environment data
 gtk_version = None
 mlt_version = None
-appversion = "2.8.0"
+appversion = "2.10.0"
 RUNNING_FROM_INSTALLATION = 0
 RUNNING_FROM_DEV_VERSION = 1
 RUNNING_FROM_FLATPAK = 2
@@ -209,17 +209,6 @@ def get_track(index):
 def timeline_visible():
     return _timeline_displayed
 
-def mlt_version_is_equal_or_greater(test_version):
-    runtime_ver = mlt_version.split(".")
-    test_ver = test_version.split(".")
-    
-    if runtime_ver[0] >= test_ver[0]:
-        if  runtime_ver[1] >= test_ver[1]:
-            if  runtime_ver[2] >= test_ver[2]:
-                return True
-    
-    return False
-
 def mlt_version_is_greater_correct(test_version):
     runtime_ver = mlt_version.split(".")
     test_ver = test_version.split(".")
@@ -236,10 +225,9 @@ def mlt_version_is_greater_correct(test_version):
     return False
 
 def runtime_version_greater_then_test_version(test_version, runtime_version):
-    runtime_ver = runtime_version.split(".")
-    test_ver = test_version.split(".")
-    
-    if runtime_ver[0] > test_ver[0]:
+    runtime_ver = list(map(int, runtime_version.split(".")))
+    test_ver = list(map(int, test_version.split(".")))
+    if int(runtime_ver[0]) > int(test_ver[0]):
         return True
     elif runtime_ver[0] == test_ver[0]:
         if runtime_ver[1] > test_ver[1]:
@@ -257,6 +245,10 @@ def set_copy_paste_objects(objs):
 def get_copy_paste_objects():
     return _copy_paste_objects
 
+def clear_copy_paste_objects():
+    global _copy_paste_objects
+    _copy_paste_objects = None
+
 def screen_size_small_height():
     if SCREEN_HEIGHT < 901:
         return True
@@ -272,6 +264,12 @@ def screen_size_small_width():
     else:
         return False
 
+def screen_size_large_width():
+    if SCREEN_WIDTH > 1670:
+        return True
+    else:
+        return False
+        
 def screen_size_small():
     if screen_size_small_height() == True or screen_size_small_width() == True:
         return True
@@ -299,12 +297,8 @@ def clear_trim_clip_cache():
 
 
 # Called from tline "motion_notify_event" when drag is not on.
-# This is only used by multitrimmode.py to have data to enter trims with keyboard correctly
+# This is only used by multitrimmode.py to have data to enter trims with keyboard correctly.
 def set_mouse_current_non_drag_pos(x, y):
     global last_mouse_x, last_mouse_y
     last_mouse_x = x
     last_mouse_y = y
-    
-
-        
-         
