@@ -23,13 +23,14 @@ This module handles the less central actions inited by user from menu.
 """
 
 
-from gi.repository import Gtk, Gdk
+from gi.repository import Gtk, GLib
 
 import threading
 import webbrowser
 import time
 
 import appconsts
+import containeractions
 import dialogs
 import dialogutils
 import editorpersistance
@@ -50,56 +51,76 @@ profile_manager_dialog = None
 # ---------------------------------------------- recreate icons
 class RecreateIconsThread(threading.Thread):
     
-    def __init__(self):
+    def __init__(self, recreate_progress_window):
         threading.Thread.__init__(self)
+        self.recreate_progress_window = recreate_progress_window
 
     def run(self):
-        Gdk.threads_enter()
-        recreate_progress_window = dialogs.recreate_icons_progress_dialog()
-        time.sleep(0.1)
-        Gdk.threads_leave()
-
-        no_icon_path = respaths.IMAGE_PATH + projectdata.FALLBACK_THUMB
         loaded = 0
         for key in PROJECT().media_files.keys():
             media_file = PROJECT().media_files[key]
-            Gdk.threads_enter()
-            recreate_progress_window.info.set_text(media_file.name)
-            Gdk.threads_leave()
+            GLib.idle_add(self._progress_window_set_text, media_file)
 
             if ((not isinstance(media_file, patternproducer.AbstractBinClip))
                 and (not isinstance(media_file, projectdata.BinColorClip))):
-                if media_file.type == appconsts.AUDIO:
-                    icon_path = respaths.IMAGE_PATH + "audio_file.png"
-                    media_file.info = None
+
+                if hasattr(media_file, "container_data") and media_file.container_data != None:
+                    # Generators
+                    if media_file.container_data.container_type == appconsts.CONTAINER_CLIP_FLUXITY:
+                        action_object = containeractions.get_action_object(media_file.container_data)
+                        media_file.icon_path, media_file.icon = action_object.re_render_screenshot()
+                        media_file.container_data.data_slots["icon_file"] = media_file.icon_path
+                    else:
+                        media_file.icon_path = None
+                        media_file.icon = None
+                        
+                        media_file.create_icon()
+                        
+                        action_object = containeractions.get_action_object(media_file.container_data)
+                        media_file.icon_path = action_object.get_container_thumbnail_path()
+                        cr, media_file.icon = action_object.create_image_surface(media_file.icon_path)
                 else:
-                    (icon_path, length, info) = projectdata.thumbnailer.write_image(media_file.path)
-                    media_file.info = info
-                media_file.icon_path = icon_path
-                media_file.create_icon()
+                    # Media files
+                    if media_file.type == appconsts.AUDIO:
+                        icon_path = respaths.IMAGE_PATH + "audio_file.png"
+                        media_file.info = None
+                    else:
+                        (icon_path, length, info) = projectdata.thumbnailer.write_image(media_file.path)
+                        media_file.info = info
+                    media_file.icon_path = icon_path
+                    media_file.create_icon()
 
             loaded = loaded + 1
-            
-            Gdk.threads_enter()
-            loaded_frac = float(loaded) / float(len(PROJECT().media_files))
-            recreate_progress_window.progress_bar.set_fraction(loaded_frac)
-            time.sleep(0.01)
-            Gdk.threads_leave()
 
-        # Update editor gui
-        Gdk.threads_enter()
-        recreate_progress_window.destroy()
+            GLib.idle_add(self._progress_window_update_fraction, loaded)
+
+        GLib.idle_add(self._progress_window_destroy)
+
+        GLib.idle_add(self._exit_update)
+
+    def _progress_window_set_text(self, media_file):
+        if self.recreate_progress_window != None:
+            self.recreate_progress_window.info.set_text(media_file.name)
+
+    def _progress_window_update_fraction(self, loaded):
+        if self.recreate_progress_window != None:
+            loaded_frac = float(loaded) / float(len(PROJECT().media_files))
+            self.recreate_progress_window.progress_bar.set_fraction(loaded_frac)
+            time.sleep(0.01)
+
+    def _progress_window_destroy(self):
+        self.recreate_progress_window.destroy()
+        self.recreate_progress_window = None
         time.sleep(0.3)
-        Gdk.threads_leave()
         
-        Gdk.threads_enter()
+    def _exit_update(self):
         gui.media_list_view.fill_data_model()
         gui.bin_list_view.fill_data_model()
-        gui.enable_save()
-        Gdk.threads_leave()
-
+        gui.enable_save(PROJECT())
+        
 def recreate_media_file_icons():
-    recreate_thread = RecreateIconsThread()
+    recreate_progress_window = dialogs.recreate_icons_progress_dialog()
+    recreate_thread = RecreateIconsThread(recreate_progress_window)
     recreate_thread.start()
 
 def show_project_info():
@@ -121,6 +142,13 @@ def quick_reference():
     except:
         dialogutils.info_message(_("Help page not found!"), _("Unfortunately the webresource containing help information\nfor this application was not found."), None)
 
+def quick_reference_web():
+    try:
+        url = "http://jliljebl.github.io/flowblade/webhelp/help.html"
+        webbrowser.open(url)
+    except:
+        dialogutils.info_message(_("Help page not found!"), _("Unfortunately the webresource containing help information\nfor this application was not found."), None)
+
 def profiles_manager():
     global profile_manager_dialog
     profile_manager_dialog = profilesmanager.profiles_manager_dialog()
@@ -149,14 +177,18 @@ def _watermark_remove_callback(button, widgets):
     file_path_value_label.set_text(_("Not Set"))
     current_sequence().remove_watermark()
       
-def toggle_fullscreen():
+def toggle_fullscreen(w=None, e=None):
     if editorpersistance.prefs.global_layout == appconsts.SINGLE_WINDOW:
         if editorstate.fullscreen == False:
            gui.editor_window.window.fullscreen()
            editorstate.fullscreen = True
+           gui.editor_window.fullscreen_press.surface = gui.editor_window.fullscreen_press.fullscreen_exit_icon
+           gui.editor_window.fullscreen_press.widget.queue_draw()
         else:
            gui.editor_window.window.unfullscreen()
            editorstate.fullscreen = False
+           gui.editor_window.fullscreen_press.surface = gui.editor_window.fullscreen_press.fullscreen_icon
+           gui.editor_window.fullscreen_press.widget.queue_draw()
     else:
         if gui.editor_window.window.has_toplevel_focus() == True:
             if editorstate.fullscreen == False:
