@@ -21,12 +21,10 @@
 """
 Module checks environment for available codecs and formats.
 """
-from gi.repository import GObject
-import mlt
-
-import dialogutils
-import editorstate
-import gui
+try:
+    import mlt7 as mlt
+except:
+    import mlt
 
 acodecs = None
 vcodecs = None
@@ -38,7 +36,7 @@ environment_detection_success = False
 
 def check_available_features(repo):
     try:
-        print "Detecting environment..."
+        print("Detecting environment...")
         global acodecs        
         global vcodecs    
         global formats
@@ -85,15 +83,14 @@ def check_available_features(repo):
         for i in range(mlt.Properties.count(envtransitions)):
             transitions[mlt.Properties.get_name(envtransitions, i)] = True
             
-        print "MLT detection succeeded, " + str(len(formats)) + " formats, "  \
-        + str(len(vcodecs)) + " video codecs and " + str(len(acodecs)) + " audio codecs found."
-        print str(len(services)) + " MLT services found."
+        print("MLT detection succeeded, " + str(len(formats)) + " formats, "  \
+        + str(len(vcodecs)) + " video codecs and " + str(len(acodecs)) + " audio codecs found.")
+        print(str(len(services)) + " MLT services found.")
 
         environment_detection_success = True
 
     except:
-        print "Environment detection failed, environment unknown."
-        GObject.timeout_add(2000, _show_failed_environment_info)
+        return
 
 def render_profile_supported(frmt, vcodec, acodec):
     if environment_detection_success == False:
@@ -108,7 +105,7 @@ def render_profile_supported(frmt, vcodec, acodec):
                 return (True, "")
             else:
                 err_msg = "format " + frmt
-                print err_msg
+                print(err_msg)
         else:
             err_msg = "video codec " + vcodec
     else:
@@ -116,15 +113,7 @@ def render_profile_supported(frmt, vcodec, acodec):
 
     return (False, err_msg)
 
-def _show_failed_environment_info():
-    primary_txt = "Environment detection failed!"
-    secondary_txt = "You will probably be presented with filters, transitions\nand rendering options that are not available on your system." + \
-    "\n---\nYou may experience sudden crashes when adding filters or\nattempting rendering." + \
-    "\n---\nYour MLT Version is: "+ editorstate.mlt_version + "\n" + \
-    "Only report this as a bug if the MLT version above is >= 0.7.6."
-    
-    dialogutils.info_message(primary_txt, secondary_txt, gui.editor_window.window)
-    return False
+
 
     
     
