@@ -23,14 +23,16 @@ Module contains objects that wrap mlt.Transition objects used to mix video betwe
 two tracks.
 """
 import copy
-import mlt
+try:
+    import mlt7 as mlt
+except:
+    import mlt
 import os
 import xml.dom.minidom
 
 import appconsts
-import compositorfades
 import mltrefhold
-import patternproducer
+import mltfilters
 import propertyparse
 import respaths
 
@@ -41,7 +43,6 @@ PROPERTY = appconsts.PROPERTY
 EXTRA_EDITOR = appconsts.EXTRA_EDITOR
 MLT_SERVICE = appconsts.MLT_SERVICE
 COMPOSITOR = "compositortransition"
-AUTO_FADE_COMPOSITOR = "autofadecompositor"
 
 # Property types.
 PROP_INT = appconsts.PROP_INT
@@ -72,64 +73,72 @@ compositors = None
 blenders = None
 autofades = None
 alpha_combiners = None
+wipe_compositors = None
+
+# these are no longer presented as options for users since 2.4
+dropped_compositors = ["##pict_in_pict", "##opacity_kf", "##dodge"]
 
 def init_module():
 
     # translations and module load order make us do this in method instead of at module load
-    global wipe_lumas, compositors, blenders, name_for_type, rendered_transitions, single_track_render_type_names, autofades, alpha_combiners
+    global wipe_lumas, compositors, blenders, name_for_type, rendered_transitions, single_track_render_type_names, autofades, alpha_combiners, wipe_compositors
     wipe_lumas = { \
-                _("Vertical From Center"):"bi-linear_x.pgm",
-                _("Vertical Top to Bottom"):"wipe_top_to_bottom.pgm",
-                _("Vertical Bottom to Top"):"wipe_bottom_to_top.pgm",
-                _("Horizontal From Center"):"bi-linear_y.pgm",
-                _("Horizontal Left to Right"):"wipe_left_to_right.pgm",
-                _("Horizontal Right to Left"):"wipe_right_to_left.pgm",
+                _("Burst"):"burst.pgm",
+                _("Checkerboard"):"checkerboard.pgm",
+                _("Circle From In"):"circle_in_to_out.pgm",
+                _("Circle From Out"):"circle_out_to_in.pgm",
                 _("Clock Left To Right"):"clock_left_to_right.pgm",
                 _("Clock Right to Left"):"clock_right_to_left.pgm",
                 _("Clock Symmetric"):"symmetric_clock.pgm",
-                _("Stripes Horizontal"):"blinds_in_to_out.pgm",
-                _("Stripes Horizontal Big"):"blinds_in_to_out_big.pgm",
-                _("Stripes Horizontal Moving"):"blinds_sliding.pgm",
-                _("Stripes Vertical"):"vertical_blinds_in_to_out.pgm",
-                _("Stripes Vertical Big"):"vertical_blinds_in_to_out_big.pgm",
-                _("Burst"):"burst.pgm",
-                _("Circle From In"):"circle_in_to_out.pgm",
-                _("Circle From Out"):"circle_out_to_in.pgm",
                 _("Cloud"):"cloud.pgm",
-                _("Hatched 1"):"hatched_1.pgm",
-                _("Hatched 2"):"hatched_2.pgm",
-                _("Hourglass"):"hourglass_1.pgm",
-                _("Puddles"):"mountains.pgm",
-                _("Rings"):"radial-bars.pgm",
-                _("Rectangle From In"):"rectangle_in_to_out.pgm",
-                _("Rectangle From Out"):"rectangle_out_to_in.pgm",
-                _("Rectangle Bars"):"square2-bars.pgm",
-                _("Sand"):"sand.pgm",
-                _("Sphere"):"sphere.pgm",
-                _("Spiral Abstract"):"spiral_abstract_1.pgm",
-                _("Spiral"):"spiral.pgm",
-                _("Spiral Galaxy"):"spiral2.pgm",
-                _("Spiral Big"):"spiral_big.pgm",
-                _("Spiral Medium"):"spiral_medium.pgm",
-                _("Spots"):"spots.pgm",
-                _("Star"):"star_2.pgm",
-                _("Arch"):"fractal_1.pgm",
-                _("Patches"):"fractal_4.pgm",
-                _("Free Stripes"):"fractal_5.pgm",
-                _("Free Curves"):"fractal_7.pgm",
+                _("Cross"):"Cross.pgm",
                 _("Diagonal 1"):"wipe_diagonal_1.pgm",
                 _("Diagonal 2"):"wipe_diagonal_2.pgm",
                 _("Diagonal 3"):"wipe_diagonal_3.pgm",
                 _("Diagonal 4"):"wipe_diagonal_4.pgm",
-                _("Checkerboard"):"checkerboard_small.pgm"}
+                _("Flower"):"flower.pgm",
+                _("Fogg"):"fogg.pgm",
+                _("Free Curves"):"free_curves.pgm",
+                _("Free Stripes"):"free_stripes.pgm",
+                _("Heart"):"heart.pgm",
+                _("Honeycomb"):"kosette_honeycomb.pgm",
+                _("Horizontal From Center"):"bi-linear_y.pgm",
+                _("Horizontal Left to Right"):"wipe_left_to_right.pgm",
+                _("Horizontal Right to Left"):"wipe_right_to_left.pgm",
+                _("Paint"):"kosette_paint.pgm",
+                _("Patches"):"fractal.pgm",
+                _("Puzzle"):"Puzzle.pgm",
+                _("Rays"):"rays.pgm",
+                _("Rectangle Bars"):"Rectangle_Bars.pgm",
+                _("Rectangle From In"):"rectangle_in_to_out.pgm",
+                _("Rectangle From Out"):"rectangle_out_to_in.pgm",
+                _("Rectangles"):"square_bars.pgm",
+                _("Rings"):"radial_bars.pgm",
+                _("Sand"):"sand.pgm",
+                _("Sphere"):"sphere.pgm",
+                _("Spiral Abstract"):"spiral_abstract.pgm",
+                _("Spiral Big"):"spiral_big.pgm",
+                _("Spiral Galaxy"):"spiral2.pgm",
+                _("Spiral Medium"):"spiral_medium.pgm",
+                _("Spiral"):"spiral.pgm",
+                _("Spots"):"spots.pgm",
+                _("Star"):"star.pgm",
+                _("Stripes Horizontal Big"):"blinds_in_to_out_big.pgm",
+                _("Stripes Horizontal"):"blinds_in_to_out.pgm",
+                _("Stripes Horizontal Moving"):"blinds_sliding.pgm",
+                _("Stripes Vertical Big"):"vertical_blinds_in_to_out_big.pgm",
+                _("Stripes Vertical"):"vertical_blinds_in_to_out.pgm",
+                _("Torn frame"):"Torn_frame.pgm",
+                _("Vertical Bottom to Top"):"wipe_bottom_to_top.pgm",
+                _("Vertical From Center"):"bi-linear_x.pgm",
+                _("Vertical Top to Bottom"):"wipe_top_to_bottom.pgm",
+                _("Wood"):"wood.pgm"}
 
     # name -> mlt_compositor_transition_infos key dict.
     unsorted_compositors = [ (_("Dissolve"),"##opacity_kf"),
                              (_("Picture in Picture"),"##pict_in_pict"),
-                             (_("Region"), "##region"),
                              (_("Affine Blend"), "##affineblend"),
                              (_("Blend"), "##blend"),
-                             (_("Wipe Clip Length"),"##wipe"),
                              (_("Transform"),"##affine")]
 
     compositors = sorted(unsorted_compositors, key=lambda comp: comp[0])   
@@ -155,18 +164,10 @@ def init_module():
                 (_("Subtract"),"##subtract"),
                 (_("Value"),"##value")]
 
-    autofades = [(_("Fade In"),"##auto_fade_in"),
-                (_("Fade Out"),"##auto_fade_out")]
     
-    alpha_combiners = [ (_("Alpha XOR"),"##alphaxor"),
-                        (_("Alpha Out"),"##alphaout"),
-                        (_("Alpha In"),"##alphain")]
 
-    """ These ain't doing correct Porter-Duff
-                        (_("Alpha Over"),"##alphaover"),
-                        (_("Alpha Atop"),"##alphaatop")]
-    """
-                        
+    wipe_compositors = [(_("Wipe Clip Length"),"##wipe")]
+
     for comp in compositors:
         name, comp_type = comp
         name_for_type[comp_type] = name
@@ -175,20 +176,13 @@ def init_module():
         name, comp_type = blend
         name_for_type[comp_type] = name
 
-    for fade in autofades:
-        name, comp_type = fade
-        name_for_type[comp_type] = name
-
-    for acomb in alpha_combiners:
-        name, comp_type = acomb
+    for wc in wipe_compositors:
+        name, comp_type = wc
         name_for_type[comp_type] = name
         
     # Rendered transition names and types
     rendered_transitions = [  (_("Dissolve"), RENDERED_DISSOLVE), 
-                              (_("Wipe"), RENDERED_WIPE),
-                              (_("Color Dip"), RENDERED_COLOR_DIP),
-                              (_("Fade In"), RENDERED_FADE_IN),
-                              (_("Fade Out"), RENDERED_FADE_OUT)]
+                              (_("Wipe"), RENDERED_WIPE)]
 
 # ------------------------------------------ compositors
 class CompositorTransitionInfo:
@@ -198,10 +192,8 @@ class CompositorTransitionInfo:
     """
     def __init__(self, compositor_node):
         self.mlt_service_id = compositor_node.getAttribute(MLT_SERVICE)
-        if compositor_node.hasAttribute(AUTO_FADE_COMPOSITOR):
-            self.auto_fade_compositor = bool(compositor_node.getAttribute(AUTO_FADE_COMPOSITOR))
-        else:
-            self.auto_fade_compositor = False
+
+        self.auto_fade_compositor = False # DEPRECATED, remove later.
 
         self.xml = compositor_node.toxml()
         self.name = compositor_node.getElementsByTagName(NAME).item(0).firstChild.nodeValue
@@ -256,6 +248,8 @@ class CompositorTransition:
             self._set_luma_service_default_values()
         elif self.info.mlt_service_id == "region":
             self._set_region_service_default_values()
+        elif self.info.mlt_service_id == "matte":
+            pass
         else:
             self._set_blend_service_default_values()
         
@@ -358,17 +352,14 @@ class CompositorObject:
         self.clip_out = out_frame
         self.transition.mlt_transition.set("in", str(in_frame))
         self.transition.mlt_transition.set("out", str(out_frame))
-        self.update_autofade_keyframes()
 
     def set_length_from_in(self, length):
         self.clip_out = self.clip_in + length - 1
         self.transition.mlt_transition.set("out", str(self.clip_out))
-        self.update_autofade_keyframes()
 
     def set_length_from_out(self, length):
         self.clip_in = self.clip_out - length + 1
         self.transition.mlt_transition.set("in", str(self.clip_in))
-        self.update_autofade_keyframes()
         
     def create_mlt_objects(self, mlt_profile):
         self.transition.create_mlt_transition(mlt_profile)
@@ -379,8 +370,9 @@ class CompositorObject:
         self.transition.properties = copy.deepcopy(source_compositor.transition.properties)
         self.transition.update_editable_mlt_properties()
 
-    def get_copy_paste_objects(self):
-        # Copy-paste is handled with tuple data (properties, mlt_service_id)
+    def get_copy_paste_data(self):
+        # Copy-paste data object is tuple (properties, mlt_service_id)
+        # This saved with type info in editorstate.py
         return (copy.deepcopy(self.transition.properties), self.transition.info.mlt_service_id)
 
     def do_values_copy_paste(self, copy_paste_data):
@@ -391,15 +383,6 @@ class CompositorObject:
         if mlt_service_id == self.transition.info.mlt_service_id:
             self.transition.properties = copy.deepcopy(properties)
             self.transition.update_editable_mlt_properties()
-        
-    def update_autofade_keyframes(self):
-        if self.transition.info.auto_fade_compositor == False:
-            return
-        
-        if self.transition.info.name == "##auto_fade_in":
-            compositorfades.set_auto_fade_in_keyframes(self)
-        else:
-            compositorfades.set_auto_fade_out_keyframes(self)
             
 # -------------------------------------------------- compositor interface methods
 def load_compositors_xml(transitions):
@@ -409,12 +392,12 @@ def load_compositors_xml(transitions):
     """
     compositors_doc = xml.dom.minidom.parse(respaths.COMPOSITORS_XML_DOC)
 
-    print "Loading transitions..."
+    print("Loading transitions...")
     compositor_nodes = compositors_doc.getElementsByTagName(COMPOSITOR)
     for c_node in compositor_nodes:
         compositor_info = CompositorTransitionInfo(c_node)
         if (not compositor_info.mlt_service_id in transitions) and len(transitions) > 0:
-            print "MLT transition " + compositor_info.mlt_service_id + " not found."
+            print("MLT transition " + compositor_info.mlt_service_id + " not found.")
             global not_found_transitions
             not_found_transitions.append(compositor_info)
             continue
@@ -423,7 +406,7 @@ def load_compositors_xml(transitions):
 
 def get_wipe_resource_path_for_sorted_keys_index(sorted_keys_index):
     # This exists to avoid sending a list of sorted keys around or having to use global variables
-    keys = wipe_lumas.keys()
+    keys = list(wipe_lumas.keys())
     keys.sort()
     return get_wipe_resource_path(keys[sorted_keys_index])
     
@@ -432,7 +415,11 @@ def get_wipe_resource_path(key):
     return respaths.WIPE_RESOURCES_PATH + img_file
 
 def create_compositor(compositor_type):
-    transition_info = mlt_compositor_transition_infos[compositor_type]
+    try:
+        transition_info = mlt_compositor_transition_infos[compositor_type]
+    except KeyError:
+        return None # Compositor was removed from MLT and cannot made available anymore.
+        
     compositor = CompositorObject(transition_info)
     compositor.compositor_index = -1 # not used since SAVEFILE = 3
     compositor.name = name_for_type[compositor_type]
@@ -455,7 +442,6 @@ def is_alpha_combiner(compositor_type_test):
     
     return False
 
-
 # ------------------------------------------------------ rendered transitions
 # These are tractor objects used to create rendered transitions.
 def get_rendered_transition_tractor(current_sequence, 
@@ -466,8 +452,7 @@ def get_rendered_transition_tractor(current_sequence,
                                     action_to_out,
                                     action_to_in,
                                     transition_type_selection_index,
-                                    wipe_luma_sorted_keys_index,
-                                    gdk_color_str):
+                                    wipe_luma_sorted_keys_index):
 
     name, transition_type = rendered_transitions[transition_type_selection_index]
     
@@ -479,12 +464,11 @@ def get_rendered_transition_tractor(current_sequence,
     current_sequence.clone_clip_and_filters(orig_from, from_clip)
 
     # New to clip
-    if not(transition_type == RENDERED_FADE_IN or transition_type == RENDERED_FADE_OUT): # fades to not use to_clip
-        if orig_to.media_type != appconsts.PATTERN_PRODUCER:
-            to_clip = current_sequence.create_file_producer_clip(orig_to.path, None, False, orig_to.ttl)# File producer
-        else:
-            to_clip = current_sequence.create_pattern_producer(orig_to.create_data) # pattern producer
-        current_sequence.clone_clip_and_filters(orig_to, to_clip)
+    if orig_to.media_type != appconsts.PATTERN_PRODUCER:
+        to_clip = current_sequence.create_file_producer_clip(orig_to.path, None, False, orig_to.ttl)# File producer
+    else:
+        to_clip = current_sequence.create_pattern_producer(orig_to.create_data) # pattern producer
+    current_sequence.clone_clip_and_filters(orig_to, to_clip)
 
     # Create tractor and tracks
     tractor = mlt.Tractor()
@@ -494,78 +478,56 @@ def get_rendered_transition_tractor(current_sequence,
     multitrack.connect(track0, 0)
     multitrack.connect(track1, 1)
 
-    # we'll set in and out points for images and pattern producers.
-    if not(transition_type == RENDERED_FADE_IN or transition_type == RENDERED_FADE_OUT): # fades to not use to_clip or some other data used here
-        if from_clip.media_type == appconsts.IMAGE or from_clip.media_type == appconsts.PATTERN_PRODUCER:
-            length = action_from_out - action_from_in
-            from_clip.clip_in = 0
-            from_clip.clip_out = length
+    # Set in and out points for images and pattern producers.
+    if from_clip.media_type == appconsts.IMAGE or from_clip.media_type == appconsts.PATTERN_PRODUCER:
+        length = action_from_out - action_from_in
+        from_clip.clip_in = 0
+        from_clip.clip_out = length
 
-        if to_clip.media_type == appconsts.IMAGE or to_clip.media_type == appconsts.PATTERN_PRODUCER:
-            length = action_to_out - action_to_in
-            to_clip.clip_in = 0
-            to_clip.clip_out = length
-    else:
-        length = action_from_out
-        if from_clip.media_type == appconsts.IMAGE or from_clip.media_type == appconsts.PATTERN_PRODUCER:
-            from_clip.clip_in = 0
-            from_clip.clip_out = length
+    if to_clip.media_type == appconsts.IMAGE or to_clip.media_type == appconsts.PATTERN_PRODUCER:
+        length = action_to_out - action_to_in
+        to_clip.clip_in = 0
+        to_clip.clip_out = length
             
     # Add clips to tracks and create keyframe string for mixing
-    if transition_type == RENDERED_DISSOLVE or transition_type == RENDERED_WIPE:
-        # Add clips. Images and pattern producers always fill full track.
-        if from_clip.media_type != appconsts.IMAGE and from_clip.media_type != appconsts.PATTERN_PRODUCER:
-            track0.insert(from_clip, 0, action_from_in, action_from_out)
-        else:
-            track0.insert(from_clip, 0, 0, action_from_out - action_from_in)
-            
-        if to_clip.media_type != appconsts.IMAGE and to_clip.media_type != appconsts.PATTERN_PRODUCER: 
-            track1.insert(to_clip, 0, action_to_in, action_to_out)
-        else:
-            track1.insert(to_clip, 0, 0,  action_to_out - action_to_in)
-        kf_str = "0=0/0:100%x100%:0.0;"+ str(tractor.get_length() - 1) + "=0/0:100%x100%:100.0"
-    elif transition_type == RENDERED_COLOR_DIP:
-        length = action_from_out - action_from_in
-        first_clip_length = length / 2
-        second_clip_length = length - first_clip_length
-        color_clip = patternproducer.create_color_producer(current_sequence.profile, gdk_color_str)
-        track0.insert(color_clip, 0, 0, length)
-        track1.insert(from_clip, 0, action_from_in, action_from_in + first_clip_length)
-        track1.insert(to_clip, 1, action_to_out - second_clip_length, action_to_out)
-        kf_str = "0=0/0:100%x100%:100.0;"+ str(first_clip_length) + "=0/0:100%x100%:0.0;" + str(tractor.get_length() - 1) + "=0/0:100%x100%:100.0"
-    elif (transition_type == RENDERED_FADE_IN or transition_type == RENDERED_FADE_OUT):
-        color_clip = patternproducer.create_color_producer(current_sequence.profile, gdk_color_str)
-        track0.insert(color_clip, 0, 0, length)
-        if transition_type ==  RENDERED_FADE_IN:
-            track1.insert(from_clip, 0, orig_from.clip_in, orig_from.clip_in + length)
-            kf_str = "0=0/0:100%x100%:0.0;"+ str(length) + "=0/0:100%x100%:100.0"
-        else: # transition_type ==  RENDERED_FADE_OUT
-            track1.insert(from_clip, 0, orig_from.clip_out - length, orig_from.clip_out)
-            kf_str = "0=0/0:100%x100%:100.0;"+ str(length) + "=0/0:100%x100%:0.0"
+    # Images and pattern producers always fill full track.
+    if from_clip.media_type != appconsts.IMAGE and from_clip.media_type != appconsts.PATTERN_PRODUCER:
+        track0.insert(from_clip, 0, action_from_in, action_from_out)
+    else:
+        track0.insert(from_clip, 0, 0, action_from_out - action_from_in)
+        
+    if to_clip.media_type != appconsts.IMAGE and to_clip.media_type != appconsts.PATTERN_PRODUCER: 
+        track1.insert(to_clip, 0, action_to_in, action_to_out)
+    else:
+        track1.insert(to_clip, 0, 0,  action_to_out - action_to_in)
 
     # Create transition
-    transition = mlt.Transition(current_sequence.profile, "region")
-    mltrefhold.hold_ref(transition)
-    transition.set("composite.geometry", str(kf_str)) # controls mix over time
-    transition.set("composite.automatic",1)
-    transition.set("composite.aligned", 0)
-    transition.set("composite.deinterlace",0)
-    transition.set("composite.distort",0)
-    transition.set("composite.fill",1)
-    transition.set("composite.operator","over")
-    transition.set("composite.luma_invert",0)
-    transition.set("composite.progressive",1)
-    transition.set("composite.softness",0)
+    kf_str = "0=0.0;"+ str(tractor.get_length() - 1) + "=1.0"
+    transition = mlt.Transition(current_sequence.profile, "frei0r.cairoblend")
+    if transition_type == RENDERED_WIPE:
+        transition.set("0", "1.0") # opacity
+    else:
+        transition.set("0", str(kf_str)) # opacity
+    transition.set("1", "normal") # blend mode
     transition.set("in", 0)
-    transition.set("out", tractor.get_length() - 1)
+    transition.set("out", int(tractor.get_length() - 1))
     transition.set("a_track", 0)
     transition.set("b_track", 1)
-
-    # Setting luma resource file turns dissolve into wipe
+    
+    # Do wipe with "shape" filter.
     if transition_type == RENDERED_WIPE:
+        kf_str = "0=0.0;"+ str(tractor.get_length() - 1) + "=100.0"
         wipe_resource_path = get_wipe_resource_path_for_sorted_keys_index(wipe_luma_sorted_keys_index)
-        transition.set("composite.luma", str(wipe_resource_path))
-
+        filter_object = mltfilters.FilterObject(mltfilters._shape_filter_info)
+        filter_object.create_mlt_filter(current_sequence.profile)
+        filter_object.mlt_filter.set(str("resource"), str(wipe_resource_path))
+        filter_object.mlt_filter.set(str("mix"), str(kf_str))
+        filter_object.mlt_filter.set(str("softness"), str(0))
+        filter_object.mlt_filter.set(str("invert"), str(0))
+        filter_object.mlt_filter.set(str("use_mix"), str(1))
+        filter_object.mlt_filter.set(str("audio_match"), str(0))
+        to_clip.attach(filter_object.mlt_filter)
+        
     # Add transition
     field = tractor.field()
     field.plant_transition(transition, 0,1)
