@@ -28,12 +28,13 @@ Rendering is done in app.player object of class mltplayer.Player
 
 from gi.repository import Gtk
 
-import mlt
+try:
+    import mlt7 as mlt
+except:
+    import mlt
 import hashlib
 import os
-import subprocess
 import time
-import threading
 
 import appconsts
 import atomicfile
@@ -41,18 +42,14 @@ import dialogutils
 import editorstate
 from editorstate import current_sequence
 from editorstate import PROJECT
-from editorstate import PLAYER
 import editorpersistance
 import gui
 import guicomponents
 import guiutils
 import jobs
 import mltprofiles
-import mltrefhold
 import renderconsumer
 import rendergui
-import respaths
-import sequence
 import userfolders
 import utils
 
@@ -77,13 +74,15 @@ transition_render_done_callback = None
 # ---------------------------------- rendering action and dialogs
 def get_args_vals_list_for_current_selections():
     profile = get_current_profile()
-    encoding_option_index = widgets.encoding_panel.encoding_selector.widget.get_active()
+    encoding_option_index = widgets.encoding_panel.encoding_selector.get_selected_encoding_index()
     quality_option_index = widgets.encoding_panel.quality_selector.widget.get_active()
-        
+    """
     if widgets.render_type_panel.type_combo.get_active() == 1: # Preset encodings                                                                             -1)
         encoding_option = renderconsumer.non_user_encodings[widgets.render_type_panel.presets_selector.widget.get_active()]
         args_vals_list = encoding_option.get_args_vals_tuples_list(profile)
-    elif widgets.args_panel.use_args_check.get_active() == False: # User encodings
+    """
+
+    if widgets.args_panel.use_args_check.get_active() == False: # User encodings
         args_vals_list = renderconsumer.get_args_vals_tuples_list_for_encoding_and_quality( profile, 
                                                                                             encoding_option_index, 
                                                                                             quality_option_index)
@@ -105,16 +104,17 @@ def get_args_vals_list_for_current_selections():
 
 def get_current_gui_selections():
     selections = {}
-    selections["use_user_encodings"] = widgets.render_type_panel.type_combo.get_active()
-    selections["encoding_option_index"] = widgets.encoding_panel.encoding_selector.widget.get_active()
+    selections["encoding_option_index"] = widgets.encoding_panel.encoding_selector.get_selected_encoding_index()
+    selections["encoding_option_name"]  = widgets.encoding_panel.encoding_selector.categorised_combo.get_selected_name() # FIXME
     selections["quality_option_index"]= widgets.encoding_panel.quality_selector.widget.get_active()
-    selections["presets_index"] = widgets.render_type_panel.presets_selector.widget.get_active()
     selections["folder"] = widgets.file_panel.out_folder.get_current_folder()
     selections["name"] = widgets.file_panel.movie_name.get_text()
     selections["range"] = widgets.range_cb.get_active()
     selections["markinmarkout"] = (PROJECT().c_seq.tractor.mark_in, PROJECT().c_seq.tractor.mark_out)
     selections["use_project_profile"] = widgets.profile_panel.use_project_profile_check.get_active()
     selections["render_profile"] = widgets.profile_panel.out_profile_combo.widget.get_active()
+    selections["render_profile_name"] = widgets.profile_panel.out_profile_combo.categories_combo.get_selected()
+    selections["audio_frequency"] = widgets.encoding_panel.sample_rate_selector.widget.get_active()
     if widgets.args_panel.use_args_check.get_active() == True:
         if widgets.args_panel.text_buffer == None:
             buf = widgets.args_panel.opts_view.get_buffer()
@@ -130,10 +130,12 @@ def get_current_gui_selections():
     return selections
 
 def set_saved_gui_selections(selections):
-    widgets.render_type_panel.type_combo.set_active(selections["use_user_encodings"])
-    widgets.encoding_panel.encoding_selector.widget.set_active(selections["encoding_option_index"])
+    try:
+        enc_op_name = selections["encoding_option_name"]
+        widgets.encoding_panel.encoding_selector.categorised_combo.set_selected(enc_op_name)
+    except:
+        print("Old style encoding option value could not be loaded.")
     widgets.encoding_panel.quality_selector.widget.set_active(selections["quality_option_index"])
-    widgets.render_type_panel.presets_selector.widget.set_active(selections["presets_index"])
     widgets.file_panel.out_folder.set_current_folder(selections["folder"])
     widgets.file_panel.movie_name.set_text(selections["name"])
     widgets.range_cb.set_active(selections["range"])
@@ -144,14 +146,15 @@ def set_saved_gui_selections(selections):
             PROJECT().c_seq.tractor.mark_in = mark_in
             PROJECT().c_seq.tractor.mark_out = mark_out
         widgets.profile_panel.use_project_profile_check.set_active(selections["use_project_profile"] )
-        widgets.profile_panel.out_profile_combo.widget.set_active(selections["render_profile"] )
+        try:
+            profile_name = selections["render_profile_name"]
+            widgets.profile_panel.out_profile_combo.categories_combo.set_selected(profile_name)
+        except:
+            print("Old style profile option value could not be loaded.")
         if selections["render_args"] != None:
             widgets.args_panel.use_args_check.set_active(True)
-            if widgets.args_panel.text_buffer == None:
-                buf = widgets.args_panel.opts_view.get_buffer()
-            else:
-                buf = widgets.args_panel.text_buffer
-            buf.set_text(selections["render_args"])
+
+        widgets.encoding_panel.sample_rate_selector.widget.set_active(selections["audio_frequency"])
     except:
         pass
     
@@ -181,13 +184,21 @@ def create_widgets():
     Widgets for editing render properties and viewing render progress.
     """
     widgets.file_panel = rendergui.RenderFilePanel()
-    widgets.render_type_panel = rendergui.RenderTypePanel(_render_type_changed, _preset_selection_changed)
+    #widgets.render_type_panel = rendergui.RenderTypePanel(_render_type_changed, _preset_selection_changed)
     widgets.profile_panel = rendergui.RenderProfilePanel(_out_profile_changed)
     widgets.encoding_panel = rendergui.RenderEncodingPanel(widgets.file_panel.extension_label)
     if (editorstate.SCREEN_HEIGHT > 898):
-        widgets.args_panel = rendergui.RenderArgsPanel(_save_opts_pressed, _load_opts_pressed,
-                                                       _display_selection_in_opts_view,
-                                                       set_default_values_for_widgets)
+        if editorstate.screen_size_large_width() == False:
+            widgets.args_panel = rendergui.RenderArgsPanel(_save_opts_pressed, _load_opts_pressed,
+                                                           _display_selection_in_opts_view,
+                                                           set_default_values_for_widgets)
+        else:
+            widgets.args_panel = rendergui.RenderArgsRow(_save_opts_pressed, _load_opts_pressed,
+                                                           _display_selection_in_opts_view,
+                                                           set_default_values_for_widgets)
+
+
+                 
     else:
         widgets.args_panel = rendergui.RenderArgsPanelSmall(_save_opts_pressed, _load_opts_pressed,
                                                             _display_selection_in_opts_view)
@@ -195,18 +206,19 @@ def create_widgets():
     # Range, Render, Reset, Render Queue
     widgets.render_button = guiutils.get_render_button()
     widgets.range_cb = rendergui.get_range_selection_combo()
-    widgets.queue_button = Gtk.Button(_("To Queue"))
+    widgets.queue_button = Gtk.Button(label=_("To Queue"))
     widgets.queue_button.set_tooltip_text(_("Save Project in Render Queue"))
-    
-    # Tooltips
-    widgets.range_cb.set_tooltip_text(_("Select render range"))
+    widgets.render_range_panel = rendergui.RenderRangePanel(widgets.range_cb)
 
+    # Add some tooltips
+    widgets.range_cb.set_tooltip_text(_("Select render range"))
     widgets.render_button.set_tooltip_text(_("Begin Rendering"))
 
 def set_default_values_for_widgets(movie_name_too=False):
     if len(renderconsumer.encoding_options) == 0:# this won't work if no encoding options available
         return                   # but we don't want crash, so that we can inform user
-    widgets.encoding_panel.encoding_selector.widget.set_active(0)
+    widgets.encoding_panel.encoding_selector.categorised_combo.refill(renderconsumer.categorized_encoding_options)
+    widgets.encoding_panel.encoding_selector.categorised_combo.set_selected(renderconsumer.DEFAULT_ENCODING_NAME)
     if movie_name_too == True:
         widgets.file_panel.movie_name.set_text("movie")
 
@@ -218,69 +230,25 @@ def set_default_values_for_widgets(movie_name_too=False):
     widgets.args_panel.use_args_check.set_active(False)
     widgets.profile_panel.use_project_profile_check.set_active(True)
 
+def update_encoding_selector():
+    # Lets make doubly sure we're not updating somthing that soes npt exist.
+    while hasattr(widgets, "encoding_panel") == False:
+        print("no encoding_panel")
+        time.sleep(0.5)
+        
+    widgets.encoding_panel.encoding_selector.categorised_combo.refill(renderconsumer.categorized_encoding_options)
+    widgets.encoding_panel.encoding_selector.categorised_combo.set_selected(renderconsumer.DEFAULT_ENCODING_NAME)
+    
 def enable_user_rendering(value):
     widgets.encoding_panel.set_sensitive(value)
     widgets.profile_panel.set_sensitive(value)
     widgets.info_panel.set_sensitive(value)
     widgets.args_panel.set_sensitive(value)
-"""
-def set_render_gui():
-    progress_window.status_label.set_text(_("<b>Output File: </b>") + get_file_path())
-    progress_window.status_label.set_use_markup(True)
-    progress_window.remaining_time_label.set_text(_("<b>Estimated time left: </b>"))
-    progress_window.remaining_time_label.set_use_markup(True)
-    progress_window.passed_time_label.set_text(_("<b>Render time: </b>"))
-    progress_window.passed_time_label.set_use_markup(True)
-    progress_window.progress_bar.set_text("0%")
-"""
+
 def save_render_start_time():
     global render_start_time
     render_start_time = time.time()
 
-"""
-def set_render_progress_gui(fraction):
-    progress_window.progress_bar.set_fraction(fraction)
-    pros = int(fraction * 100)
-    progress_window.progress_bar.set_text(str(pros) + "%")
-
-    try:
-        passed_time = time.time() - render_start_time
-        full_time_est = (1.0 / fraction) * passed_time
-        passed_str = utils.get_time_str_for_sec_float(passed_time)
-        progress_window.passed_time_label.set_text(_("<b>Render Time: </b>") + passed_str)
-        progress_window.passed_time_label.set_use_markup(True)
-
-        if pros > 0.99: # Only start giving estimations after rendering has gone on for a while.
-            left_est = full_time_est - passed_time
-            left_str = utils.get_time_str_for_sec_float(left_est)
-            progress_window.remaining_time_label.set_text(_("<b>Estimated Time Left: </b>") + left_str)
-            progress_window.remaining_time_label.set_use_markup(True)
-
-    except: # A fraction of 0 usually gets sent here at beginning of rendering
-        pass
-        
-def exit_render_gui():
-    if aborted == True:
-        print("render aborted")
-        return
-
-    global progress_window
-
-    set_render_progress_gui(1.0)
-    passed_time = time.time() - render_start_time
-    passed_str = utils.get_time_str_for_sec_float(passed_time)
-    print("render done, time: " + passed_str)
-    
-    progress_window.remaining_time_label.set_text(_("<b>Estimated Time Left: </b>"))
-    progress_window.remaining_time_label.set_use_markup(True)
-    progress_window.passed_time_label.set_text(_("<b>Render Time: </b>") + passed_str)
-    progress_window.passed_time_label.set_use_markup(True)
-    progress_window.progress_bar.set_text(_("Render Complete!"))
-    
-    dialogutils.delay_destroy_window(progress_window, 2.0)
-    progress_window = None
-"""
-
 def maybe_open_rendered_file_in_bin():
     if widgets.args_panel.open_in_bin.get_active() == False:
         return
@@ -289,62 +257,44 @@ def maybe_open_rendered_file_in_bin():
     open_media_file_callback(file_path)
 
 def get_current_profile():
-    profile_index = widgets.profile_panel.out_profile_combo.widget.get_active()
-    if profile_index == 0:
-        # project_profile is first selection in combo box
-        profile = PROJECT().profile
-    else:
-        profile = mltprofiles.get_profile_for_index(profile_index - 1)
+    profile_desc = widgets.profile_panel.out_profile_combo.categories_combo.get_selected()
+    profile = mltprofiles.get_profile(profile_desc)
     return profile
 
 def fill_out_profile_widgets():
     """
     Called some time after widget creation when current_sequence is known and these can be filled.
     """
-    widgets.profile_panel.out_profile_combo.fill_options()
+    
+    widgets.profile_panel.out_profile_combo.set_initial_selection()
     _fill_info_box(current_sequence().profile)
 
 def reload_profiles():
     renderconsumer.load_render_profiles()
-    fill_out_profile_widgets()
+    
+    widgets.profile_panel.out_profile_combo.categories_combo.refill(mltprofiles._categorized_profiles)
+    widgets.profile_panel.out_profile_combo.set_initial_selection()
 
-def _render_type_changed():
-    if widgets.render_type_panel.type_combo.get_active() == 0: # User Defined
-        enable_user_rendering(True)
-        set_default_values_for_widgets()
-        widgets.render_type_panel.presets_selector.widget.set_sensitive(False)
-        _preset_selection_changed()
-        widgets.encoding_panel.encoding_selector.encoding_selection_changed()
-    else: # Preset Encodings
-        enable_user_rendering(False)
-        widgets.render_type_panel.presets_selector.widget.set_sensitive(True)
-        _preset_selection_changed()
-        if editorstate.screen_size_small_height() == False:
-            widgets.args_panel.opts_view.set_sensitive(False)
-            widgets.args_panel.opts_view.get_buffer().set_text("")
-
-def _out_profile_changed():
-    selected_index = widgets.profile_panel.out_profile_combo.widget.get_active()
-    if selected_index == 0:
-        _fill_info_box(current_sequence().profile)
-    else:
-        profile = mltprofiles.get_profile_for_index(selected_index - 1)
-        _fill_info_box(profile)
+def _out_profile_changed(categories_combo):
+    # FIXME: 'out_profile_combo' is actually the panel containing the combobox
+    try:
+        profile_desc = widgets.profile_panel.out_profile_combo.categories_combo.get_selected()
+    except TypeError:
+        # We are getting events here when refilling the profile widget after user has updated profiles
+        # and it is easier to ignore the events then to disconnect the listener.
+        return 
+    profile = mltprofiles.get_profile(profile_desc)
+    _fill_info_box(profile)
 
 def _fill_info_box(profile):
     info_panel = guicomponents.get_profile_info_small_box(profile)
     widgets.info_panel = info_panel
     widgets.profile_panel.out_profile_info_box.display_info(info_panel)
 
-def _preset_selection_changed():
-    enc_index = widgets.render_type_panel.presets_selector.widget.get_active()
-    ext = renderconsumer.non_user_encodings[enc_index].extension
-    widgets.file_panel.extension_label.set_text("." + ext)
-   
 def _display_selection_in_opts_view():
     profile = get_current_profile()
     widgets.args_panel.display_encoding_args(profile,
-                                             widgets.encoding_panel.encoding_selector.widget.get_active(), 
+                                             widgets.encoding_panel.encoding_selector.get_selected_encoding_index(), 
                                              widgets.encoding_panel.quality_selector.widget.get_active())
     
 def _save_opts_pressed():
@@ -396,13 +346,9 @@ def _render_frame_buffer_clip_dialog_callback(dialog, response_id, fb_widgets, m
             dialogutils.warning_message(primary_txt, secondary_txt, dialog)
             return
             
-        profile_index = fb_widgets.out_profile_combo.get_active()
-        if profile_index == 0:
-            # project_profile is first selection in combo box
-            profile = PROJECT().profile
-        else:
-            profile = mltprofiles.get_profile_for_index(profile_index - 1)
-        profile_desc = profile.description().replace(" ", "_")
+        profile_desc = fb_widgets.categories_combo.get_selected()
+        profile = mltprofiles.get_profile(profile_desc)
+        profile_desc = profile_desc.replace(" ", "_")
     
         encoding_option_index = fb_widgets.encodings_cb.get_active()
         quality_option_index = fb_widgets.quality_cb.get_active()
@@ -481,14 +427,10 @@ def _render_reverse_clip_dialog_callback(dialog, response_id, fb_widgets, media_
             return
 
         # Profile
-        profile_index = fb_widgets.out_profile_combo.get_active()
-        if profile_index == 0:
-            # project_profile is first selection in combo box
-            profile = PROJECT().profile
-        else:
-            profile = mltprofiles.get_profile_for_index(profile_index - 1)
-        profile_desc = profile.description().replace(" ", "_")
-        
+        profile_desc = fb_widgets.categories_combo.get_selected()
+        profile = mltprofiles.get_profile(profile_desc)
+        profile_desc = profile_desc.replace(" ", "_")
+
         # Render consumer properties
         encoding_option_index = fb_widgets.encodings_cb.get_active()
         quality_option_index = fb_widgets.quality_cb.get_active()
@@ -539,7 +481,7 @@ def _render_reverse_clip_dialog_callback(dialog, response_id, fb_widgets, media_
         dialog.destroy()
 
 def _REVERSE_render_stop(dialog, response_id):
-    print("reverse clip render done")
+    print("Reverse clip render done")
 
     global motion_renderer, motion_progress_update
     motion_renderer.running = False
@@ -582,7 +524,7 @@ def render_single_track_transition_clip(transition_producer, encoding_option_ind
     progress_bar = Gtk.ProgressBar()
     dialog = rendergui.clip_render_progress_dialog(_transition_render_stop, title, window_text, progress_bar, gui.editor_window.window)
     
-    motion_progress_update = renderconsumer.ProgressWindowThread(dialog, progress_bar, motion_renderer, _transition_render_stop)
+    motion_progress_update = guiutils.ProgressWindowThread(dialog, progress_bar, motion_renderer, _transition_render_stop)
     motion_progress_update.start()
 
 def _transition_render_stop(dialog, response_id):
